# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import IPloneSiteRoot
from collective.quickupload.browser.quickupload_settings import IQuickUploadControlPanel
from plone.app.layout.viewlets import common
from zope.component import getUtility


class QuickUploadViewlet(common.ViewletBase):

    def render(self):
        portal = getUtility(IPloneSiteRoot)
        try:
            qup_prefs = IQuickUploadControlPanel(portal)
        except AttributeError:
            return ""
        if not qup_prefs.show_upload_action:
            return ""
        return self.index()



"""Main product initializer
"""
from zope.i18n import MessageFactory
import logging
import pkg_resources

try:
    pkg_resources.get_distribution('plone.dexterity')
except pkg_resources.DistributionNotFound:
    HAS_DEXTERITY = False
else:
    HAS_DEXTERITY = True

logger = logging.getLogger("collective.quickupload")
siteMessageFactory = MessageFactory("collective.quickupload")

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



import mock
import unittest2 as unittest


class TestCase(unittest.TestCase):

    def test_MissingExtension(self):
        from collective.quickupload.browser.uploadcapable import MissingExtension
        self.assertTrue(issubclass(MissingExtension, Exception))

    def test_get_id_from_filename__without_extension(self):
        context = mock.Mock()
        context.getCharset.return_value = 'utf-8'
        from collective.quickupload.browser.uploadcapable import MissingExtension
        from collective.quickupload.browser.uploadcapable import get_id_from_filename
        filename = 'FILENAME'
        self.assertRaises(
            MissingExtension,
            lambda: get_id_from_filename(filename, context)
        )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five.browser import BrowserView
from collective.quickupload.browser.quickupload_settings import IQuickUploadControlPanel
from collective.quickupload.portlet.quickuploadportlet import IQuickUploadPortlet
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRetriever
from ua_parser import user_agent_parser
from zope.component import getMultiAdapter
from zope.component import getUtilitiesFor
from zope.component import getUtility


class QuickuploadHelper(BrowserView):

    def show_quickupload_action(self):
        portal = getUtility(IPloneSiteRoot)
        qup_prefs = IQuickUploadControlPanel(portal)
        if not qup_prefs.show_upload_action:
            return False
        context_state = getMultiAdapter(
            (aq_inner(self.context), self.request),
            name=u'plone_context_state')
        if not (
                context_state.is_default_page() or
                context_state.is_folderish()):
            return False
        # If the Quickuploader portlet is shown in the current context, don't
        # show the Upload action, since the portlet takes precedence
        portlet_managers = [
            x[1] for x in getUtilitiesFor(IPortletManager)
            if not x[0].startswith('plone.dashboard')
        ]
        for mgr in portlet_managers:
            retriever = getMultiAdapter((self.context, mgr), IPortletRetriever)
            portlets = retriever.getPortlets()
            for portlet in portlets:
                if IQuickUploadPortlet.providedBy(portlet["assignment"]):
                    return False
        return True


def can_dnd(user_agent):
    """All modern browsers are able to upload files"""
    result = user_agent_parser.Parse(user_agent)
    if result['user_agent']['family'] == u'IE':
        return int(result['user_agent']['major']) > 9
    return True



# -*- coding: utf-8 -*-
## Copyright (C)2010 Alter Way Solutions
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.Archetypes.utils import shasattr
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.FactoryTool import TempFolder
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Sessions.SessionDataManager import SessionDataManagerErr
from collective.quickupload import logger
from collective.quickupload import siteMessageFactory as _
from collective.quickupload.interfaces import IQuickUploadCapable
from collective.quickupload.interfaces import IQuickUploadNotCapable
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.layout.globals.interfaces import IViewView
from plone.app.portlets.portlets import base
from plone.memoize.compress import xhtml_compress
from plone.portlets.interfaces import IPortletDataProvider
from zope import schema
from zope.formlib import form
from zope.i18nmessageid import MessageFactory
from zope.interface import implements
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
PMF = MessageFactory('plone')


def isTemporary(obj):
    """Check to see if an object is temporary"""
    if not shasattr(obj, 'isTemporary'):
        return False
    if obj.isTemporary():
        return False

    parent = aq_base(aq_parent(aq_inner(obj)))
    return hasattr(parent, 'meta_type') \
        and parent.meta_type == TempFolder.meta_type


JAVASCRIPT = """
  // workaround this MSIE bug :
  // https://dev.plone.org/plone/ticket/10894
  if (jQuery.browser.msie) jQuery("#settings").remove();
  var Browser = {};
  Browser.onUploadComplete = function() {
      window.location.reload();
  }
  loadUploader = function() {
      var ulContainer = jQuery('.QuickUploadPortlet .uploaderContainer');
      ulContainer.each(function(){
          var uploadUrl =  jQuery('.uploadUrl', this).val();
          var uploadData =  jQuery('.uploadData', this).val();
          var UlDiv = jQuery(this);
          jQuery.ajax({
                     type: 'GET',
                     url: uploadUrl,
                     data: uploadData,
                     dataType: 'html',
                     contentType: 'text/html; charset=utf-8',
                     success: function(html) {
                        if (html.indexOf('quick-uploader') != -1) {
                            UlDiv.html(html);
                        }
                     } });
      });
  }
  jQuery(document).ready(loadUploader);
"""


class IQuickUploadPortlet(IPortletDataProvider):
    """Quickupload portlet schema
    """

    header = schema.TextLine(
        title=_(u"Box title"),
        default=u"",
        description=_(u"If title is empty, the portlet title will be the "
                      u"Media Choice + ' Quick Upload'."),
        required=False)

    upload_portal_type = schema.Choice(
        title=_(u"Content type"),
        description=_(u"Choose the portal type used for file upload. "
                      u"Let the default configuration for an automatic portal "
                      u"type, depending on settings defined in "
                      u"content_type_registry."),
        required=True,
        default='auto',
        vocabulary="collective.quickupload.fileTypeVocabulary")

    upload_media_type = schema.Choice(
        title=_(u"Media type"),
        description=_(u"Choose the media type used for file upload. "
                      u"image, audio, video ..."),
        required=False,
        default='',
        vocabulary=SimpleVocabulary(
            [SimpleTerm('', '', _(u"All")),
             SimpleTerm('image', 'image', _(u"Images")),
             SimpleTerm('video', 'video', _(u"Video files")),
             SimpleTerm('audio', 'audio', _(u"Audio files")),
             SimpleTerm('flash', 'flash', _(u"Flash files"))]
        ),
    )


class Assignment(base.Assignment):
    """Portlet assignment.
    """

    implements(IQuickUploadPortlet)

    def __init__(self, header='', upload_portal_type='auto',
                 upload_media_type=''):
        self.header = header
        self.upload_portal_type = upload_portal_type
        self.upload_media_type = upload_media_type

    @property
    def title(self):
        """portlet title
        """
        if self.header:
            return PMF(self.header)

        media = self.upload_media_type
        if not media or '*.' in media:
            return _(u'Files Quick Upload')
        elif media == 'image':
            return _(u'Images Quick Upload')
        else:
            return _(u'label_media_quickupload',
                     default='${medialabel} Quick Upload',
                     mapping={'medialabel': media.capitalize()})


class Renderer(base.Renderer):
    """Portlet renderer.
    """

    _template = ViewPageTemplateFile('quickuploadportlet.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)
        context = aq_inner(self.context)
        self.ploneview = context.restrictedTraverse('@@plone')
        self.pm = getToolByName(context, 'portal_membership')

    def _clean_session(self):
        request = self.request
        try:
            session = request.get('SESSION', None)
        except SessionDataManagerErr:
            logger.debug(
                'Error occurred getting session data. Falling back to request.'
            )
            session = None
        # empty typeupload and mediaupload session
        # since the portlet don't use it, but another app could
        if session:
            # session does not implement dict API. it's a TransientObject
            if session.has_key('typeupload'):
                session.delete('typeupload')
            if session.has_key('mediaupload'):
                session.delete('mediaupload')

    def render(self):
        self._clean_session()
        return xhtml_compress(self._template())

    @property
    def available(self):
        if not (IViewView.providedBy(self.view)
                or IFolderContentsView.providedBy(self.view)):
            return False

        context = aq_inner(self.context)

        if not IQuickUploadCapable.providedBy(context):
            return False
        elif IQuickUploadNotCapable.providedBy(context):
            return False
        elif not self.pm.checkPermission('Add portal content', context):
            return False
        elif isTemporary(context):
            return False

        upload_portal_type = self.data.upload_portal_type
        if (upload_portal_type and upload_portal_type != 'auto'
                and upload_portal_type not in [
                    t.id for t in self.context.getAllowedTypes()
                ]):
            return False
        else:
            return True

    def getUploadUrl(self):
        """
        return upload url
        in current folder
        """
        folder_url = self.ploneview.getCurrentFolderUrl()
        return '%s/@@quick_upload' % folder_url

    def getDataForUploadUrl(self):
        data_url = ''
        if self.data.upload_portal_type != 'auto':
            data_url += 'typeupload=%s&' % self.data.upload_portal_type
        if self.data.upload_media_type:
            data_url += 'mediaupload=%s' % self.data.upload_media_type
        return data_url

    def javascript(self):
        return JAVASCRIPT


class AddForm(base.AddForm):
    """Portlet add form.
    """
    form_fields = form.Fields(IQuickUploadPortlet)

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    """Portlet edit form.
    """
    form_fields = form.Fields(IQuickUploadPortlet)



# -*- coding: utf-8 -*-
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import login
from plone.app.testing import setRoles

import doctest


class QuickUploadLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import collective.quickupload
        self.loadZCML(package=collective.quickupload)
        try:
            import plone.namedfile
            self.loadZCML(package=plone.namedfile)
        except:
            pass

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        self.applyProfile(portal, 'collective.quickupload:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        portal.invokeFactory('Folder', 'test-folder', title=u"Test Folder")
        setRoles(portal, TEST_USER_ID, ['Member'])


QUICKUPLOAD_FIXTURE = QuickUploadLayer()

QUICKUPLOAD_INTEGRATION_TESTING = IntegrationTesting(
    bases=(QUICKUPLOAD_FIXTURE,),
    name='collective.quickupload:Integration',
)

QUICKUPLOAD_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(QUICKUPLOAD_FIXTURE,),
    name='collective.quickupload:Functional',
)

OPTIONFLAGS = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)



from zope.interface import Interface


class IQuickUploadFileSetter(Interface):
    """Adapter to set file data on a content
    """

    def set(self, data, content_type):
        """sets data of a content type on adaptated content
        @return an error code or '' if success
        """


class IQuickUploadCapable(Interface):
    """Any container/object which supports quick uploading
    """


class IQuickUploadNotCapable(Interface):
    """Any container/object which NEVER supports quick uploading
    """


# Copy of zope.filepresentation.interfaces.IFileFactory.
# We can't inherit from this because otherwise this adapter will be registered
# for IFileFactory too, and will be used for PUT requests (FTP/WebDAV)
class IDefaultFileFactory(Interface):

    def __call__(name, content_type, data):
        """Create a file

        where a file is an object with adapters to `IReadFile`
        and `IWriteFile`.

        The file `name`, content `type`, and `data` are provided to help
        create the object.
        """


class IQuickUploadFileFactory(IDefaultFileFactory):
    """used for QuickUploadFileFactory
    """


class IQuickUploadFileUpdater(IDefaultFileFactory):
    """used for QuickUploadFileFactory
    """



# -*- coding: utf-8 -*-

import unittest2 as unittest

from zope.component import getMultiAdapter

from plone.app.testing import TEST_USER_ID
from plone.app.testing import logout
from plone.app.testing import setRoles

from collective.quickupload.testing import QUICKUPLOAD_INTEGRATION_TESTING


class ControlPanelTest(unittest.TestCase):

    layer = QUICKUPLOAD_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.cp = getattr(self.portal, 'portal_controlpanel')

    def test_installed(self):
        # entry is in the control panel
        installed = [a.getAction(self)['id'] for a in self.cp.listActions()]
        self.failUnless('QuickUpload' in installed)

    def test_uninstalled(self):
        setup_tool = getattr(self.portal, 'portal_setup')
        setup_tool.runAllImportStepsFromProfile('profile-collective.quickupload:uninstall')

        # entry is removed from the control panel
        installed = [a.getAction(self)['id'] for a in self.cp.listActions()]
        self.failUnless('QuickUpload' in installed)

    def test_view(self):
        view = getMultiAdapter((self.portal, self.portal.REQUEST),
                               name='quickupload-controlpanel')
        view = view.__of__(self.portal)
        self.failUnless(view())

    def test_view_protected(self):
        # view can not be accessed by anonymous users
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(Unauthorized,
                          self.portal.restrictedTraverse,
                          '@@quickupload-controlpanel')


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



from Products.CMFCore.utils import getToolByName
from collective.quickupload import logger
from collective.quickupload.interfaces import IQuickUploadFileSetter
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.file import NamedFile
from plone.namedfile.file import NamedImage
from plone.namedfile.interfaces import INamedFileField
from plone.namedfile.interfaces import INamedImageField
from plone.rfc822.interfaces import IPrimaryField
from zope.component import adapts
from zope.interface import implements
from zope.schema import getFieldsInOrder

try:
    from plone.namedfile.file import NamedBlobFile
    from plone.namedfile.file import NamedBlobImage
    from plone.namedfile.interfaces import INamedBlobFileField
    from plone.namedfile.interfaces import INamedBlobImageField
    HAVE_BLOBS = True
except:
    HAVE_BLOBS = False


class DexterityFileSetter(object):
    implements(IQuickUploadFileSetter)
    adapts(IDexterityContent)

    def __init__(self, context):
        self.context = context

    def set(self, data, filename, content_type):
        error = ''
        obj = self.context
        ttool = getToolByName(obj, 'portal_types')
        ctype = ttool[obj.portal_type]
        schema = ctype.lookupSchema()
        fields = getFieldsInOrder(schema)
        file_fields = [field for name, field in fields
                       if INamedFileField.providedBy(field)
                       or INamedImageField.providedBy(field)]
        if len(file_fields) == 0:
            error = u'serverError'
            logger.info("An error happens : the dexterity content type %s "
                        "has no file field, rawdata can't be created",
                        obj.absolute_url())
        for file_field in file_fields:
            if IPrimaryField.providedBy(file_field):
                break
        else:
            # Primary field can't be set ttw,
            # then, we take the first one
            file_field = file_fields[0]

        # TODO: use adapters
        if HAVE_BLOBS and INamedBlobImageField.providedBy(file_field):
            value = NamedBlobImage(
                data=data, contentType=content_type,
                filename=unicode(filename, 'utf-8')
            )
        elif HAVE_BLOBS and INamedBlobFileField.providedBy(file_field):
            value = NamedBlobFile(
                data=data, contentType=content_type,
                filename=unicode(filename, 'utf-8')
            )
        elif INamedImageField.providedBy(file_field):
            value = NamedImage(
                data=data, contentType=content_type,
                filename=unicode(filename, 'utf-8')
            )
        elif INamedFileField.providedBy(file_field):
            value = NamedFile(
                data=data, contentType=content_type,
                filename=unicode(filename, 'utf-8')
            )

        file_field.set(obj, value)
        return error



# -*- coding: utf-8 -*-
#
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
from AccessControl import Unauthorized
from Acquisition import aq_inner
from Products.Archetypes.event import ObjectEditedEvent
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from collective.quickupload import logger
from collective.quickupload import siteMessageFactory as _
from collective.quickupload.interfaces import IQuickUploadCapable
from collective.quickupload.interfaces import IQuickUploadFileFactory
from collective.quickupload.interfaces import IQuickUploadFileSetter
from collective.quickupload.interfaces import IQuickUploadFileUpdater
from plone.i18n.normalizer.interfaces import IIDNormalizer
from thread import allocate_lock
from zope import component
from zope import interface
from zope.lifecycleevent import ObjectModifiedEvent
from zope.event import notify

import transaction

try:
    from zope.app.container.interfaces import INameChooser
except ImportError:
    from zope.container.interfaces import INameChooser


upload_lock = allocate_lock()


class MissingExtension(Exception):
    """Exception when the filename has no extension."""


def get_id_from_filename(filename, context):
    charset = getattr(context, 'getCharset', None) and context.getCharset()\
        or 'utf-8'
    name = filename.decode(charset).rsplit('.', 1)
    if len(name) != 2:
        raise MissingExtension('It seems like the file extension is missing.')
    normalizer = component.getUtility(IIDNormalizer)
    newid = '.'.join((normalizer.normalize(name[0]), name[1]))
    newid = newid.replace('_', '-').replace(' ', '-').lower()
    return newid


class QuickUploadCapableFileFactory(object):
    interface.implements(IQuickUploadFileFactory)
    component.adapts(IQuickUploadCapable)

    def __init__(self, context):
        self.context = aq_inner(context)

    def __call__(self, filename, title, description, content_type, data,
                 portal_type):
        context = aq_inner(self.context)
        error = ''
        result = {}
        result['success'] = None
        newid = get_id_from_filename(filename, context)
        chooser = INameChooser(context)
        newid = chooser.chooseName(newid, context)
        # consolidation because it's different upon Plone versions
        if not title:
            # try to split filenames because we don't want
            # big titles without spaces
            title = filename.rsplit('.', 1)[0]\
                .replace('_', ' ')\
                .replace('-', ' ')

        if newid in context:
            # only here for flashupload method since a check_id is done
            # in standard uploader - see also XXX in quick_upload.py
            raise NameError, 'Object id %s already exists' % newid
        else:
            upload_lock.acquire()
            try:
                transaction.begin()
                try:
                    context.invokeFactory(type_name=portal_type, id=newid,
                                          title=title, description=description)
                except Unauthorized:
                    error = u'serverErrorNoPermission'
                except ConflictError:
                    # rare with xhr upload / happens sometimes with flashupload
                    error = u'serverErrorZODBConflict'
                except ValueError:
                    error = u'serverErrorDisallowedType'
                except Exception, e:
                    error = u'serverError'
                    logger.exception(e)

                if error:
                    if error == u'serverError':
                        logger.info(
                            "An error happens with setId from filename, "
                            "the file has been created with a bad id, "
                            "can't find %s", newid)
                else:
                    obj = getattr(context, newid)
                    if obj:
                        error = IQuickUploadFileSetter(obj).set(
                            data, filename, content_type
                        )
                        obj._at_rename_after_creation = False
                        try:
                            # Archetypes
                            obj.processForm()
                        except AttributeError:
                            # Dexterity
                            notify(ObjectModifiedEvent(obj))
                        else:
                            del obj._at_rename_after_creation

                #@TODO : rollback if there has been an error
                transaction.commit()
            finally:
                upload_lock.release()

        result['error'] = error
        if not error:
            result['success'] = obj

        return result


class QuickUploadCapableFileUpdater(object):
    interface.implements(IQuickUploadFileUpdater)
    component.adapts(IQuickUploadCapable)

    def __init__(self, context):
        self.context = aq_inner(context)

    def __call__(self, obj, filename, title, description, content_type, data):
        result = {}
        result['success'] = None

        # consolidation because it's different upon Plone versions
        if title:
            obj.setTitle(title)

        if description:
            obj.setDescription(description)

        error = IQuickUploadFileSetter(obj).set(data, filename, content_type)
        # notify edited instead of modified whereas version history is not
        # saved
        notify(ObjectEditedEvent(obj))
        obj.reindexObject()

        result['error'] = error
        if not error:
            result['success'] = obj
            IStatusMessage(obj.REQUEST).addStatusMessage(
                _('msg_file_replaced',
                  default=u"${filename} file has been replaced",
                  mapping={'filename': filename}),
                type)

        return result



from Products.Archetypes.interfaces import IBaseObject
from collective.quickupload import logger
from collective.quickupload.interfaces import IQuickUploadFileSetter
from zope.component import adapts
from zope.interface import implements


class ArchetypesFileSetter(object):
    implements(IQuickUploadFileSetter)
    adapts(IBaseObject)

    def __init__(self, context):
        self.context = context

    def set(self, data, filename, content_type):
        error = ''
        obj = self.context
        primaryField = obj.getPrimaryField()
        if primaryField is not None:
            mutator = primaryField.getMutator(obj)
            # mimetype arg works with blob files
            mutator(data, content_type=content_type, mimetype=content_type)
            if not obj.getFilename():
                obj.setFilename(filename)

            obj.reindexObject()
        else:
            # some products remove the 'primary' attribute on ATFile or ATImage
            # (which is very bad)
            error = u'serverError'
            logger.info(
                "An error happens : impossible to get the primary field "
                "for file %s, rawdata can't be created",
                obj.absolute_url()
            )

        return error



# -*- coding: utf-8 -*-
""" Vocabularies used by control panel or widget
"""
from Products.CMFCore.utils import getToolByName
from collective.quickupload import HAS_DEXTERITY
from collective.quickupload import siteMessageFactory as _
from collective.quickupload.browser.quick_upload import _listTypesForInterface
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

if HAS_DEXTERITY:
    from plone.dexterity.interfaces import IDexterityFTI


def _infoDictForType(portal, ptype):
    """
    UI type infos
    @param ptype: a portal type name
    @return: {'portal_type': xxx, 'type_ui_info': UI type info}
    """

    portal_types = getToolByName(portal, 'portal_types')
    type_info = getattr(portal_types, ptype)
    title = type_info.Title()
    if HAS_DEXTERITY and IDexterityFTI.providedBy(type_info):
        product = type_info.klass
    else:
        product = type_info.product
    type_ui_info = ("%s (portal type: %s, product: %s)" %
                    (portal.translate(title, default=title), ptype, product))
    return {
        'portal_type': ptype,
        'type_ui_info': type_ui_info
    }


class UploadFileTypeVocabulary(object):
    """Vocabulary factory for file type upload
    """
    implements(IVocabularyFactory)

    def __call__(self, context):
        context = getattr(context, 'context', context)
        portal = getToolByName(context, 'portal_url').getPortalObject()
        items = [
            SimpleTerm(
                'auto', 'auto', context.translate(_(
                    'label_default_portaltype_configuration',
                    default=u'Default configuration (Content Type Registry).'))
            )
        ]

        flt = [
            _infoDictForType(portal, tipe) for tipe in
            _listTypesForInterface(portal, 'file')
        ]
        ilt = [
            _infoDictForType(portal, tipe) for tipe in
            _listTypesForInterface(portal, 'image')
        ]
        items.extend([
            SimpleTerm(
                t['portal_type'], t['portal_type'], t['type_ui_info']
            )
            for t in flt
        ])
        file_types = [t['portal_type'] for t in flt]
        items.extend([
            SimpleTerm(
                t['portal_type'], t['portal_type'], t['type_ui_info']
            )
            for t in ilt if t['portal_type'] not in file_types
        ])

        return SimpleVocabulary(items)


UploadFileTypeVocabularyFactory = UploadFileTypeVocabulary()



from Products.CMFCore.utils import getToolByName


def v1_v2(context):
    upgrade_resources(context)


def v2_v3(context):
    upgrade_resources(context)


def upgrade_resources(context):
    context.runImportStepFromProfile('profile-collective.quickupload:default',
                                     'jsregistry')
    context.runImportStepFromProfile('profile-collective.quickupload:default',
                                     'cssregistry')
    getToolByName(context, 'portal_css').cookResources()
    getToolByName(context, 'portal_javascripts').cookResources()


def v3_v4(context):
    #Add property to quickupload property sheet
    ptool = getToolByName(context, 'portal_properties')
    qu_props = ptool.get('quickupload_properties')
    qu_props._setProperty('use_flash_as_fallback', False, 'boolean')


def v4_v5(context):
    #Add property to quickupload property sheet
    ptool = getToolByName(context, 'portal_properties')
    qu_props = ptool.get('quickupload_properties')
    if not qu_props.hasProperty('show_upload_action'):
        qu_props._setProperty('show_upload_action', False, 'boolean')
    context.runImportStepFromProfile(
        'profile-collective.quickupload:default', 'actions')

def v5_v6(context):
    #Add property to quickupload property sheet
    ptool = getToolByName(context, 'portal_properties')
    qu_props = ptool.get('quickupload_properties')
    if not qu_props.hasProperty('object_override'):
        qu_props._setProperty('object_override', False, 'boolean')




# -*- coding: utf-8 -*-
import unittest2 as unittest

from zope.interface import Interface
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.dexterity.content import Item
from plone.dexterity.fti import DexterityFTI, register
from plone.namedfile import field
from plone.namedfile.interfaces import INamedFile, INamedImage
try:
    from plone.namedfile.interfaces import INamedBlobFile, INamedBlobImage
    HAVE_BLOBS = True
except:
    HAVE_BLOBS = False

from collective.quickupload.testing import QUICKUPLOAD_INTEGRATION_TESTING
from collective.quickupload.interfaces import IQuickUploadFileSetter
from collective.quickupload.dexterity import DexterityFileSetter


FILENAME_TESTSTRING = u"Iñtërnâtiônàlizætiøn Ξεσκεπάζω tükörfúrógép äß \
いろはにほへとちりぬるを イロハニホヘト הקליטה łódź чащах มนุษย์ yağız şoföre \
::_-^°*#',;<>{}[]¡“¶¢|¿≠å+.§$%&()=?€¥±–…∞µ~∫√≈ƒ©ªº@•π⁄Ω†®∑«»\""


class ISchemaWithNamedFile(Interface):
    field = field.NamedFile()


class ISchemaWithNamedImage(Interface):
    field = field.NamedImage()


if HAVE_BLOBS:
    class ISchemaWithNamedBlobFile(Interface):
        field = field.NamedBlobFile()

    class ISchemaWithNamedBlobImage(Interface):
        field = field.NamedBlobImage()


class TestCase(unittest.TestCase):

    layer = QUICKUPLOAD_INTEGRATION_TESTING

    @property
    def portal(self):
        return self.layer['portal']

    def getFileSetter(self, schema):
        portal_type = 'testtype'
        setRoles(self.portal, TEST_USER_ID, ('Manager',))

        # prepare a fti
        fti = DexterityFTI(portal_type)
        fti.klass = 'plone.dexterity.content.Item'
        fti.schema = schema.__identifier__
        self.portal.portal_types._setObject(portal_type, fti)
        register(fti)

        # prepare an item
        item = Item('testitem')
        item.portal_type = portal_type
        self.portal._setObject(item.id, item)
        item = self.portal[item.id]

        return IQuickUploadFileSetter(item)

    def test_adapting(self):
        item = Item()
        fileSetter = IQuickUploadFileSetter(item)
        self.failUnless(fileSetter.__class__ is DexterityFileSetter)

    def test_namedfile_setting(self):
        data = 'datattatatata'
        filename = FILENAME_TESTSTRING + u".xlsx"
        content_type = 'application/vnd.ms-excel'

        fileSetter = self.getFileSetter(ISchemaWithNamedFile)
        fileSetter.set(data, filename.encode('utf-8'), content_type)
        field = fileSetter.context.field

        self.assertTrue(INamedFile.providedBy(field))
        self.assertEqual(field.filename, filename)
        self.assertEqual(field.contentType, content_type)
        self.assertEqual(field.data, data)

    def test_namedimage_setting(self):
        data = 'datattatatata'
        filename = FILENAME_TESTSTRING + u".jpeg"
        content_type = 'image/jpeg'

        fileSetter = self.getFileSetter(ISchemaWithNamedImage)
        fileSetter.set(data, filename.encode('utf-8'), content_type)
        field = fileSetter.context.field

        self.assertTrue(INamedImage.providedBy(field))
        self.assertEqual(field.filename, filename)
        self.assertEqual(field.contentType, content_type)
        self.assertEqual(field.data, data)

    if HAVE_BLOBS:
        def test_namedblobfile_setting(self):
            data = 'datattatatata'
            filename = FILENAME_TESTSTRING + u".xlsx"
            content_type = 'application/vnd.ms-excel'

            fileSetter = self.getFileSetter(ISchemaWithNamedBlobFile)
            fileSetter.set(data, filename.encode('utf-8'), content_type)
            field = fileSetter.context.field

            self.assertTrue(INamedBlobFile.providedBy(field))
            self.assertEqual(field.filename, filename)
            self.assertEqual(field.contentType, content_type)
            self.assertEqual(field.data, data)

        def test_namedblobimage_setting(self):
            data = 'datattatatata'
            filename = FILENAME_TESTSTRING + u".jpeg"
            content_type = 'image/jpeg'

            fileSetter = self.getFileSetter(ISchemaWithNamedBlobImage)
            fileSetter.set(data, filename.encode('utf-8'), content_type)
            field = fileSetter.context.field

            self.assertTrue(INamedBlobImage.providedBy(field))
            self.assertEqual(field.filename, filename)
            self.assertEqual(field.contentType, content_type)
            self.assertEqual(field.data, data)



from zope.interface import Interface, implements
from zope.component import adapts
from zope.schema import Bool, Int
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFCore.utils import getToolByName
from zope.formlib.form import FormFields
from plone.app.controlpanel.form import ControlPanelForm
from collective.quickupload import siteMessageFactory as _


class IQuickUploadControlPanel(Interface):
    """
    fields for quick upload control panel
    """
    use_flashupload = Bool(
        title=_(u"title_use_flashupload", default=u"Use Flash Upload"),
        description=_(
            u"description_use_flashupload",
            default=u"By default, the upload script is a javascript only "
                    u"tool. Check this option to replace it with a Flash "
                    u"Upload based script. For modern browsers the javascript "
                    u"tool is more powerful. Flash Upload is just more user "
                    u"friendly under other browsers (MSIE 7, MSIE 8), but has "
                    u"many problems : don't work in https, don't work behind "
                    u"HTTP Authentication ..."),
        default=False,
        required=False)

    use_flash_as_fallback = Bool(
        title=_(u"title_use_flash_fallback",
                default=u"Use flash as fallback for IE"),
        description=_(
            u"description_use_flash_fallback",
            default=u"Check if you want to use flash uload as fallback for IE"
        ),
        default=False,
        required=False)

    auto_upload = Bool(
        title=_(u"title_auto_upload", default=u"Automatic upload on select"),
        description=_(
            u"description_auto_upload",
            default=u"Check if you want to start the files upload on select, "
                    u"without submit the form. Note that you cannot choose "
                    u"file titles or descriptions with this option set to "
                    u"True."),
        default=False,
        required=False)

    show_upload_action = Bool(
        title=_('title_show_upload_action', default=u'Show "Upload" action'),
        description=_(
            'description_show_upload_action', default=u'Check if '
            'you want to have an "Upload" link in your edit bar. Clicking it '
            'will open a panel for uploading below the title of the current '
            'item. This panel is an alternative to the "Upload" portlet and '
            'does not offer any further configuration, such as filtering by '
            'content type. The panel will only be shown where the "Upload" '
            'portlet is not present.'
        ),
        default=False,
        required=False,
    )

    fill_titles = Bool(
        title=_(u"title_fill_titles", default=u"Fill title before upload"),
        description=_(
            u"description_fill_titles",
            default=u"If checked, you can fill the files titles "
                    u"before upload. Uncheck if you don't need titles."),
        default=True,
        required=False)

    fill_descriptions = Bool(
        title=_(
            u"title_fill_descriptions",
            default=u"Fill description before upload"),
        description=_(
            u"description_fill_descriptions",
            default=u"If checked, you can fill the files descriptions "
                    u"before upload. Uncheck if you don't need descriptions."),
        default=False,
        required=False)

    size_limit = Int(
        title=_(u"title_size_limit", default=u"Size limit"),
        description=_(
            u"description_size_limit",
            default=u"Size limit for each file in KB, 0 = no limit"
        ),
        default=0,
        required=True
    )

    sim_upload_limit = Int(
        title=_(
            u"title_sim_upload_limit",
            default=u"Simultaneous uploads limit"
        ),
        description=_(
            u"description_sim_upload_limit",
            default=u"Number of simultaneous files uploaded, over this number "
                    u"uploads are placed in a queue, 0 = no limit"
        ),
        default=2,
        required=True)

    object_override = Bool(
        title=_(u"title_object_override", default=u"Override by upload file"),
        description=_(
            u"description_object_override",
            default=u"If the folder already has same file name, "
                    u"Checked: Override, Non-Checked: raise upload error."),
        default=True,
        required=False)


class QuickUploadControlPanelAdapter(SchemaAdapterBase):

    adapts(IPloneSiteRoot)
    implements(IQuickUploadControlPanel)

    def __init__(self, context):
        super(QuickUploadControlPanelAdapter, self).__init__(context)
        pprop = getToolByName(context, 'portal_properties')
        self.quProps = pprop.quickupload_properties

    def get_use_flashupload(self):
        return self.quProps.getProperty('use_flashupload')

    def set_use_flashupload(self, value):
        self.quProps._updateProperty('use_flashupload', value)

    use_flashupload = property(get_use_flashupload, set_use_flashupload)

    def get_use_flash_as_fallback(self):
        return self.quProps.getProperty('use_flash_as_fallback')

    def set_use_flash_as_fallback(self, value):
        self.quProps._updateProperty('use_flash_as_fallback', value)

    use_flash_as_fallback = property(get_use_flash_as_fallback,
                                     set_use_flash_as_fallback)

    def get_auto_upload(self):
        return self.quProps.getProperty('auto_upload')

    def set_auto_upload(self, value):
        self.quProps._updateProperty('auto_upload', value)

    auto_upload = property(get_auto_upload, set_auto_upload)

    def get_show_upload_action(self):
        return self.quProps.getProperty('show_upload_action')

    def set_show_upload_action(self, value):
        self.quProps._updateProperty('show_upload_action', value)

    show_upload_action = property(
        get_show_upload_action, set_show_upload_action)

    def get_fill_titles(self):
        return self.quProps.getProperty('fill_titles')

    def set_fill_titles(self, value):
        self.quProps._updateProperty('fill_titles', value)

    fill_titles = property(get_fill_titles, set_fill_titles)

    def get_fill_descriptions(self):
        return self.quProps.getProperty('fill_descriptions')

    def set_fill_descriptions(self, value):
        self.quProps._updateProperty('fill_descriptions', value)

    fill_descriptions = property(get_fill_descriptions, set_fill_descriptions)

    def get_size_limit(self):
        return self.quProps.getProperty('size_limit')

    def set_size_limit(self, value):
        self.quProps._updateProperty('size_limit', value)

    size_limit = property(get_size_limit, set_size_limit)

    def get_sim_upload_limit(self):
        return self.quProps.getProperty('sim_upload_limit')

    def set_sim_upload_limit(self, value):
        self.quProps._updateProperty('sim_upload_limit', value)

    sim_upload_limit = property(get_sim_upload_limit, set_sim_upload_limit)

    def get_object_override(self):
        return self.quProps.getProperty('object_override')

    def set_object_override(self, value):
        self.quProps._updateProperty('object_override', value)

    object_override = property(get_object_override, set_object_override)


class QuickUploadControlPanel(ControlPanelForm):

    label = _("Quick Upload settings")
    description = _("""Control how the quick upload tool is used.""")
    form_name = _("Quick Upload settings")
    form_fields = FormFields(IQuickUploadControlPanel)



# -*- coding: utf-8 -*-
import logging
log = logging.getLogger('collective.quickupload.interfaces')
log.warning(
    "Importing interfaces from collective.quickupload.browser.interfaces is "
    "deprecated, please import from collective.quickupload.interfaces"
)

from collective.quickupload.interfaces import IQuickUploadCapable  # nopep8
from collective.quickupload.interfaces import IQuickUploadNotCapable  # nopep8
from collective.quickupload.interfaces import IQuickUploadFileFactory  # nopep8



# -*- coding: utf-8 -*-
from plone.testing import layered
from collective.quickupload import testing

import doctest
import unittest2 as unittest

functional = [
    'installation.txt',
    'controlpanel.txt',
    'quickupload_view.txt',
    'portlet.txt',
]


def test_suite():

    tests = []

    for f in functional:
        tests.append(
            layered(
                doctest.DocFileSuite('tests/{0}'.format(f),
                                     package='collective.quickupload',
                                     optionflags=testing.OPTIONFLAGS,
                                     ),
                layer=testing.QUICKUPLOAD_FUNCTIONAL_TESTING,
            )
        )

    return unittest.TestSuite(tests)



# code taken from collective.uploadify
# for the flashupload
# with many ameliorations
from AccessControl import SecurityManagement
from Acquisition import aq_inner, aq_parent
from Products.ATContentTypes.interfaces import IFileContent
from Products.ATContentTypes.interfaces import IImageContent
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Sessions.SessionDataManager import SessionDataManagerErr
from ZPublisher.HTTPRequest import HTTPRequest
from collective.quickupload import HAS_DEXTERITY
from collective.quickupload import logger
from collective.quickupload import siteMessageFactory as _
from collective.quickupload.browser.quickupload_settings import IQuickUploadControlPanel
from collective.quickupload.browser.uploadcapable import MissingExtension
from collective.quickupload.browser.uploadcapable import get_id_from_filename
from collective.quickupload.browser.utils import can_dnd
from collective.quickupload.interfaces import IQuickUploadFileFactory
from collective.quickupload.interfaces import IQuickUploadFileUpdater
from collective.quickupload.interfaces import IQuickUploadNotCapable
from plone.i18n.normalizer.interfaces import IUserPreferredFileNameNormalizer
from zope.component import getUtility
from zope.i18n import translate
from zope.schema import getFieldsInOrder
from zope.security.interfaces import Unauthorized

import mimetypes
import os
import random
import ticket as ticketmod
import urllib


import pkg_resources
try:
    pkg_resources.get_distribution('plone.uuid')
    from plone.uuid.interfaces import IUUID
    HAS_UUID = True
except pkg_resources.DistributionNotFound:
    HAS_UUID = False

if HAS_DEXTERITY:
    from plone.dexterity.interfaces import IDexterityFTI
    from plone.namedfile.interfaces import INamedFileField
    from plone.namedfile.interfaces import INamedImageField

try:
    # Python 2.6+
    import json
except ImportError:
    # Python 2.4 (Plone 3.3)
    import simplejson as json


def decodeQueryString(QueryString):
    """decode *QueryString* into a dictionary, as ZPublisher would do"""
    r = HTTPRequest(
        None,
        {'QUERY_STRING': QueryString, 'SERVER_URL': '', },
        None,
        1
    )
    r.processInputs()
    return r.form


def getDataFromAllRequests(request, dataitem):
    """Sometimes data is send using POST METHOD and QUERYSTRING
    """
    data = request.form.get(dataitem, None)
    if data is None:
        # try to get data from QueryString
        data = decodeQueryString(request.get('QUERY_STRING', '')).get(dataitem)
    return data


def find_user(context, userid):
    """Walk up all of the possible acl_users to find the user with the
    given userid.
    """

    track = set()

    acl_users = aq_inner(getToolByName(context, 'acl_users'))
    path = '/'.join(acl_users.getPhysicalPath())
    logger.debug('Visited acl_users "%s"' % path)
    track.add(path)

    user = acl_users.getUserById(userid)
    while user is None and acl_users is not None:
        context = aq_parent(aq_parent(aq_inner(acl_users)))
        acl_users = aq_inner(getToolByName(context, 'acl_users'))
        if acl_users is not None:
            path = '/'.join(acl_users.getPhysicalPath())
            logger.debug('Visited acl_users "%s"' % path)
            if path in track:
                logger.warn('Tried searching an already visited acl_users, '
                            '"%s".  All visited are: %r' % (path, list(track)))
                break
            track.add(path)
            user = acl_users.getUserById(userid)

    if user is not None:
        user = user.__of__(acl_users)

    return user


def _listTypesForInterface(context, iftype='file'):
    """
    List of portal types that have File or Image interface
    @param portal: context
    @param iftype:
        Type of interface to query for, can be either 'file' or 'image'
    @return: ['Image', 'News Item']
    """

    archetype_tool = getToolByName(context, 'archetype_tool', None)
    types_tool = getToolByName(context, 'portal_types')

    all_types = []
    if archetype_tool:
        if iftype == 'image':
            interface = IImageContent
        else:
            interface = IFileContent
        #plone4
        try:
            all_types = [
                tipe.getId() for tipe in
                archetype_tool.listPortalTypesWithInterfaces([interface])
            ]
        #plone3
        except:
            all_types = archetype_tool.listRegisteredTypes(inProject=True)
            all_types = [
                tipe['portal_type'] for tipe in all_types
                if interface.isImplementedByInstancesOf(tipe['klass'])
            ]

    if HAS_DEXTERITY:
        if iftype == 'image':
            interface = INamedImageField
        else:
            interface = INamedFileField
        for fti in types_tool.objectValues():
            if IDexterityFTI.providedBy(fti):
                fields = getFieldsInOrder(fti.lookupSchema())
                for fieldname, field in fields:
                    if interface.providedBy(field):
                        all_types.append(fti.getId())
                        break

    # fix for bug in listRegisteredTypes which returns 2 'ATFolder'
    # when asking for IBaseFolder interface
    unik_types = dict.fromkeys(all_types).keys()
    return unik_types


class QuickUploadView(BrowserView):
    """ The Quick Upload View
    """

    template = ViewPageTemplateFile("quick_upload.pt")

    def __init__(self, context, request):
        super(QuickUploadView, self).__init__(context, request)
        self.uploader_id = self._uploader_id()

    def __call__(self):
        if IQuickUploadNotCapable.providedBy(self.context):
            raise Unauthorized

        # disable diazo themes
        self.request.response.setHeader('X-Theme-Disabled', 'True')

        return self.template()

    def header_upload(self):
        request = self.request
        try:
            session = request.get('SESSION', {})
            medialabel = session.get(
                'mediaupload',
                request.get('mediaupload', 'files')
            )
        except SessionDataManagerErr:
            logger.debug('Error occurred getting session data. '
                         'Falling back to request.')
            medialabel = request.get('mediaupload', 'files')

        # to improve
        if '*.' in medialabel:
            medialabel = ''
        if not medialabel:
            return _('Files Quick Upload')
        elif medialabel == 'image':
            return _('Images Quick Upload')
        else:
            return _('label_media_quickupload',
                     default='${medialabel} Quick Upload',
                     mapping={'medialabel': translate(
                         medialabel.capitalize(),
                         domain='collective.quickupload',
                         context=self.request)
                     })

    def _uploader_id(self):
        return 'uploader%s' % str(random.random()).replace('.', '')

    def script_content(self):
        context = aq_inner(self.context)
        return context.restrictedTraverse('@@quick_upload_init')(
            for_id=self.uploader_id
        )

    def can_drag_and_drop(self):
        user_agent = self.request.get_header('User-Agent')
        return can_dnd(user_agent)


XHR_UPLOAD_JS = """
    var fillTitles = %(ul_fill_titles)s;
    var fillDescriptions = %(ul_fill_descriptions)s;
    var auto = %(ul_auto_upload)s;
    addUploadFields_%(ul_id)s = function(file, id) {
        var uploader = xhr_%(ul_id)s;
        PloneQuickUpload.addUploadFields(uploader, uploader._element, file, id, fillTitles, fillDescriptions);
    }
    sendDataAndUpload_%(ul_id)s = function() {
        var uploader = xhr_%(ul_id)s;
        PloneQuickUpload.sendDataAndUpload(uploader, uploader._element, '%(typeupload)s');
    }
    clearQueue_%(ul_id)s = function() {
        var uploader = xhr_%(ul_id)s;
        PloneQuickUpload.clearQueue(uploader, uploader._element);
    }
    onUploadComplete_%(ul_id)s = function(id, fileName, responseJSON) {
        var uploader = xhr_%(ul_id)s;
        PloneQuickUpload.onUploadComplete(uploader, uploader._element, id, fileName, responseJSON);
    }
    createUploader_%(ul_id)s= function(){
        xhr_%(ul_id)s = new qq.FileUploader({
            element: jQuery('#%(ul_id)s')[0],
            action: '%(context_url)s/@@quick_upload_file',
            autoUpload: auto,
            onAfterSelect: addUploadFields_%(ul_id)s,
            onComplete: onUploadComplete_%(ul_id)s,
            allowedExtensions: %(ul_file_extensions_list)s,
            sizeLimit: %(ul_xhr_size_limit)s,
            simUploadLimit: %(ul_sim_upload_limit)s,
            template: '<div class="qq-uploader">' +
                      '<div class="qq-upload-drop-area"><span>%(ul_draganddrop_text)s</span></div>' +
                      '<div class="qq-upload-button">%(ul_button_text)s</div>' +
                      '<ul class="qq-upload-list"></ul>' +
                      '</div>',
            fileTemplate: '<li>' +
                    '<a class="qq-upload-cancel" href="#">&nbsp;</a>' +
                    '<div class="qq-upload-infos"><span class="qq-upload-file"></span>' +
                    '<span class="qq-upload-spinner"></span>' +
                    '<span class="qq-upload-failed-text">%(ul_msg_failed)s</span></div>' +
                    '<div class="qq-upload-size"></div>' +
                '</li>',
            messages: {
                serverError: "%(ul_error_server)s",
                serverErrorAlreadyExists: "%(ul_error_already_exists)s {file}",
                serverErrorZODBConflict: "%(ul_error_zodb_conflict)s {file}, %(ul_error_try_again)s",
                serverErrorNoPermission: "%(ul_error_no_permission)s",
                serverErrorDisallowedType: "%(ul_error_disallowed_type)s",
                typeError: "%(ul_error_bad_ext)s {file}. %(ul_error_onlyallowed)s {extensions}.",
                sizeError: "%(ul_error_file_large)s {file}, %(ul_error_maxsize_is)s {sizeLimit}.",
                emptyError: "%(ul_error_empty_file)s {file}, %(ul_error_try_again_wo)s",
                missingExtension: "%(ul_error_empty_extension)s {file}"
            }
        });
    }
    jQuery(document).ready(createUploader_%(ul_id)s);
"""

FLASH_UPLOAD_JS = """
    var fillTitles = %(ul_fill_titles)s;
    var fillDescriptions = %(ul_fill_descriptions)s;
    var autoUpload = %(ul_auto_upload)s;
    clearQueue_%(ul_id)s = function() {
        jQuery('#%(ul_id)s').uploadifyClearQueue();
    }
    addUploadifyFields_%(ul_id)s = function(event, data ) {
        if ((fillTitles || fillDescriptions) && !autoUpload)  {
            var labelfiletitle = jQuery('#uploadify_label_file_title').val();
            var labelfiledescription = jQuery('#uploadify_label_file_description').val();
            jQuery('#%(ul_id)sQueue .uploadifyQueueItem').each(function() {
                ID = jQuery(this).attr('id').replace('%(ul_id)s','');
                if (!jQuery('.uploadField' ,this).length) {
                  jQuery('.cancel' ,this).after('\
                          <input type="hidden" \
                                 class="file_id_field" \
                                 name="file_id" \
                                 value ="'  + ID + '" /> \
                  ');
                  if (fillDescriptions) jQuery('.cancel' ,this).after('\
                      <div class="uploadField">\
                          <label>' + labelfiledescription + ' : </label> \
                          <textarea rows="2" \
                                 class="file_description_field" \
                                 id="description_' + ID + '" \
                                 name="description" \
                                 value="" />\
                      </div>\
                  ');
                  if (fillTitles) jQuery('.cancel' ,this).after('\
                      <div class="uploadField">\
                          <label>' + labelfiletitle + ' : </label> \
                          <input type="text" \
                                 class="file_title_field" \
                                 id="title_' + ID + '" \
                                 name="title" \
                                 value="" />\
                      </div>\
                  ');
                }
            });
        }
        if (!autoUpload) return showButtons_%(ul_id)s();
        return 'ok';
    }
    showButtons_%(ul_id)s = function() {
        if (jQuery('#%(ul_id)sQueue .uploadifyQueueItem').length) {
            jQuery('.uploadifybuttons').show();
            return 'ok';
        }
        return false;
    }
    sendDataAndUpload_%(ul_id)s = function() {
        QueueItems = jQuery('#%(ul_id)sQueue .uploadifyQueueItem');
        nbItems = QueueItems.length;
        QueueItems.each(function(i){
            filesData = {};
            ID = jQuery('.file_id_field',this).val();
            if (fillTitles && !autoUpload) {
                filesData['title'] = jQuery('.file_title_field',this).val();
            }
            if (fillDescriptions && !autoUpload) {
                filesData['description'] = jQuery('.file_description_field',this).val();
            }
            jQuery('#%(ul_id)s').uploadifySettings('scriptData', filesData);
            jQuery('#%(ul_id)s').uploadifyUpload(ID);
        })
    }
    onAllUploadsComplete_%(ul_id)s = function(event, data){
        if (!data.errors) {
           Browser.onUploadComplete();
        }
        else {
           msg= data.filesUploaded + '%(ul_msg_some_sucess)s' + data.errors + '%(ul_msg_some_errors)s';
           alert(msg);
        }
    }
    jQuery(document).ready(function() {
        jQuery('#%(ul_id)s').uploadify({
            'uploader'      : '%(portal_url)s/++resource++quickupload_static/uploader.swf',
            'script'        : '%(context_url)s/@@flash_upload_file',
            'cancelImg'     : '%(portal_url)s/++resource++quickupload_static/cancel.png',
            'folder'        : '%(physical_path)s',
            'onAllComplete' : onAllUploadsComplete_%(ul_id)s,
            'auto'          : autoUpload,
            'multi'         : true,
            'simUploadLimit': %(ul_sim_upload_limit)s,
            'sizeLimit'     : '%(ul_size_limit)s',
            'fileDesc'      : '%(ul_file_description)s',
            'fileExt'       : '%(ul_file_extensions)s',
            'buttonText'    : '%(ul_button_text)s',
            'scriptAccess'  : 'sameDomain',
            'hideButton'    : false,
            'onSelectOnce'  : addUploadifyFields_%(ul_id)s,
            'scriptData'    : {'ticket' : '%(ticket)s', 'typeupload' : '%(typeupload)s'}
        });
    });
"""


class QuickUploadInit(BrowserView):
    """ Initialize uploadify js
    """

    def __init__(self, context, request):
        super(QuickUploadInit, self).__init__(context, request)
        self.context = aq_inner(context)
        portal = getUtility(IPloneSiteRoot)
        self.qup_prefs = IQuickUploadControlPanel(portal)

    def ul_content_types_infos(self, mediaupload):
        """
        return some content types infos depending on mediaupload type
        mediaupload could be 'image', 'video', 'audio' or any
        extension like '*.doc'
        """
        ext = '*.*;'
        extlist = []
        msg = u'Choose files to upload'
        if mediaupload == 'image':
            ext = '*.jpg;*.jpeg;*.gif;*.png;'
            msg = _(u'Choose images to upload')
        elif mediaupload == 'video':
            ext = '*.flv;*.avi;*.wmv;*.mpg;'
            msg = _(u'Choose video files to upload')
        elif mediaupload == 'audio':
            ext = '*.mp3;*.wav;*.ogg;*.mp4;*.wma;*.aif;'
            msg = _(u'Choose audio files to upload')
        elif mediaupload == 'flash':
            ext = '*.swf;'
            msg = _(u'Choose flash files to upload')
        elif mediaupload:
            # you can also pass a list of extensions in mediaupload request var
            # with this syntax '*.aaa;*.bbb;'
            ext = mediaupload
            msg = _(u'Choose file for upload: ${ext}', mapping={'ext': ext})

        try:
            extlist = [
                f.split('.')[1].strip() for f in ext.split(';') if f.strip()
            ]
        except:
            extlist = []
        if extlist == ['*']:
            extlist = []

        return (ext, extlist, self._translate(msg))

    def _translate(self, msg):
        return translate(msg, context=self.request)

    def upload_settings(self):
        context = aq_inner(self.context)
        request = self.request
        try:
            session = request.get('SESSION', {})
            mediaupload = session.get(
                'mediaupload', request.get('mediaupload', ''))
            typeupload = session.get(
                'typeupload', request.get('typeupload', ''))
        except SessionDataManagerErr:
            logger.debug(
                'Error occurred getting session data. Falling back to request.'
            )
            mediaupload = request.get('mediaupload', '')
            typeupload = request.get('typeupload', '')
        portal_url = getToolByName(context, 'portal_url')()
        # use a ticket for authentication (used for flashupload only)
        ticket = context.restrictedTraverse('@@quickupload_ticket')()

        settings = dict(
            ticket                 = ticket,
            portal_url             = portal_url,
            typeupload             = '',
            context_url            = context.absolute_url(),
            physical_path          = "/".join(context.getPhysicalPath()),
            ul_id                  = self.uploader_id,
            ul_fill_titles         = self.qup_prefs.fill_titles and 'true' or 'false',
            ul_fill_descriptions   = self.qup_prefs.fill_descriptions and 'true' or 'false',
            ul_auto_upload         = self.qup_prefs.auto_upload and 'true' or 'false',
            ul_size_limit          = self.qup_prefs.size_limit and str(self.qup_prefs.size_limit*1024) or '',
            ul_xhr_size_limit      = self.qup_prefs.size_limit and str(self.qup_prefs.size_limit*1024) or '0',
            ul_sim_upload_limit    = str(self.qup_prefs.sim_upload_limit),
            ul_object_override     = self.qup_prefs.object_override and 'true' or 'false',
            ul_button_text         = self._translate(_(u'Browse')),
            ul_draganddrop_text    = self._translate(_(u'Drag and drop files to upload')),
            ul_msg_all_sucess      = self._translate(_(u'All files uploaded with success.')),
            ul_msg_some_sucess     = self._translate(_(u' files uploaded with success, ')),
            ul_msg_some_errors     = self._translate(_(u" uploads return an error.")),
            ul_msg_failed          = self._translate(_(u"Failed")),
            ul_error_try_again_wo  = self._translate(_(u"please select files again without it.")),
            ul_error_try_again     = self._translate(_(u"please try again.")),
            ul_error_empty_file    = self._translate(_(u"Selected elements contain an empty file or a folder:")),
            ul_error_empty_extension = self._translate(_(u"This file has no extension:")),
            ul_error_file_large    = self._translate(_(u"This file is too large:")),
            ul_error_maxsize_is    = self._translate(_(u"maximum file size is:")),
            ul_error_bad_ext       = self._translate(_(u"This file has invalid extension:")),
            ul_error_onlyallowed   = self._translate(_(u"Only allowed:")),
            ul_error_no_permission = self._translate(_(u"You don't have permission to add this content in this place.")),
            ul_error_disallowed_type = self._translate(_(u"This type of element is not allowed in this folder.",)),
            ul_error_already_exists = self._translate(_(u"This file already exists with the same name on server:")),
            ul_error_zodb_conflict = self._translate(_(u"A data base conflict error happened when uploading this file:")),
            ul_error_server        = self._translate(_(u"Server error, please contact support and/or try again.")),
        )

        settings['typeupload'] = typeupload
        if typeupload:
            imageTypes = _listTypesForInterface(context, 'image')
            if typeupload in imageTypes:
                ul_content_types_infos = self.ul_content_types_infos('image')
            else:
                ul_content_types_infos = self.ul_content_types_infos(
                    mediaupload
                )
        else:
            ul_content_types_infos = self.ul_content_types_infos(mediaupload)

        settings['ul_file_extensions'] = ul_content_types_infos[0]
        settings['ul_file_extensions_list'] = str(ul_content_types_infos[1])
        settings['ul_file_description'] = ul_content_types_infos[2]

        return settings

    def use_flash_as_fallback(self):
        # Use flash as fallback if xhr multiupload is not available
        # Currently this affects only IE < 10
        user_agent = self.request.get_header('User-Agent')
        fallback = self.qup_prefs.use_flash_as_fallback

        return not can_dnd(user_agent) and fallback

    def __call__(self, for_id="uploader"):
        self.uploader_id = for_id
        settings = self.upload_settings()

        if self.qup_prefs.use_flashupload or self.use_flash_as_fallback():
            return FLASH_UPLOAD_JS % settings
        return XHR_UPLOAD_JS % settings


class QuickUploadAuthenticate(BrowserView):
    """
    base view for quick upload authentication
    ticket is used only with flash upload
    nothing is done with xhr upload.
    Note: we don't use the collective.uploadify method
    for authentication because sending cookie in all requests
    is not secure.
    """
    def __init__(self, context, request):
        self.context = context
        self.request = request
        portal = getUtility(IPloneSiteRoot)
        self.qup_prefs = IQuickUploadControlPanel(portal)
        self.use_flashupload = self.qup_prefs.use_flashupload

    def _auth_with_ticket(self):
        """
        with flashupload authentication is done using a ticket
        """

        context = aq_inner(self.context)
        request = self.request
        url = context.absolute_url()

        ticket = getDataFromAllRequests(request, 'ticket')
        if ticket is None:
            raise Unauthorized('No ticket specified')

        logger.info('Authenticate using ticket, the ticket is "%s"' % ticket)
        username = ticketmod.ticketOwner(url, ticket)
        if username is None:
            logger.info('Ticket "%s" was invalidated, cannot be used '
                        'any more.' % str(ticket))
            raise Unauthorized('Ticket is not valid')

        self.old_sm = SecurityManagement.getSecurityManager()
        user = find_user(context, username)
        SecurityManagement.newSecurityManager(self.request, user)
        logger.info('Switched to user "%s"' % username)


class QuickUploadFile(QuickUploadAuthenticate):
    """ Upload a file
    """

    def __call__(self):
        """
        """
        if self.use_flashupload:
            return self.flash_upload_file()
        return self.quick_upload_file()

    def flash_upload_file(self):

        context = aq_inner(self.context)
        request = self.request
        self._auth_with_ticket()

        file_name = request.form.get("Filename", "")
        file_data = request.form.get("Filedata", None)
        content_type = mimetypes.guess_type(file_name)[0]
        portal_type = request.form.get('typeupload', '')
        title = request.form.get("title", None)
        description = request.form.get("description", None)

        if not portal_type:
            ctr = getToolByName(context, 'content_type_registry')
            portal_type = ctr.findTypeName(
                file_name.lower(), content_type, '') or 'File'

        if file_data:
            factory = IQuickUploadFileFactory(context)
            logger.debug(
                "Uploading file with flash: filename=%s, title=%s, "
                "description=%s, content_type=%s, portal_type=%s" % (
                    file_name, title, description, content_type, portal_type))

            try:
                f = factory(
                    file_name,
                    title,
                    description,
                    content_type,
                    file_data,
                    portal_type
                )
            except:
                raise
            if f['success'] is not None:
                o = f['success']
                logger.info("file url: %s" % o.absolute_url())
                SecurityManagement.setSecurityManager(self.old_sm)
                return o.absolute_url()

    def quick_upload_file(self):
        context = aq_inner(self.context)
        request = self.request
        response = request.RESPONSE

        response.setHeader('Expires', 'Sat, 1 Jan 2000 00:00:00 GMT')
        response.setHeader('Cache-control', 'no-cache')
        # the good content type woul be text/json or text/plain but IE
        # do not support it
        response.setHeader('Content-Type', 'text/html; charset=utf-8')
        # disable diazo themes
        request.response.setHeader('X-Theme-Disabled', 'True')

        if request.HTTP_X_REQUESTED_WITH:
            # using ajax upload
            file_name = urllib.unquote(request.HTTP_X_FILE_NAME)
            upload_with = "XHR"
            try:
                file = request.BODYFILE
                file_data = file.read()
                file.seek(0)
            except AttributeError:
                # in case of cancel during xhr upload
                logger.error("Upload of %s has been aborted", file_name)
                # not really useful here since the upload block
                # is removed by "cancel" action, but
                # could be useful if someone change the js behavior
                return json.dumps({u'error': u'emptyError'})
            except:
                logger.error(
                    "Error when trying to read the file %s in request",
                    file_name
                )
                return json.dumps({u'error': u'serverError'})
        else:
            # using classic form post method (MSIE<=8)
            file = request.get("qqfile", None)
            file_data = file.read()
            file.seek(0)
            filename = getattr(file, 'filename', '')
            file_name = filename.split("\\")[-1]
            try:
                file_name = file_name.decode('utf-8')
            except UnicodeDecodeError:
                pass

            file_name = IUserPreferredFileNameNormalizer(
                self.request
            ).normalize(file_name)
            upload_with = "CLASSIC FORM POST"
            # we must test the file size in this case (no client test)
            if not self._check_file_size(file_data):
                logger.info("Test file size: the file %s is too big, upload "
                            "rejected" % filename)
                return json.dumps({u'error': u'sizeError'})

        # overwrite file
        try:
            newid = get_id_from_filename(file_name, context)
        except MissingExtension:
            return json.dumps({u'error': u'missingExtension'})

        if newid in context or file_name in context:
            updated_object = context.get(newid, False) or context[file_name]
            mtool = getToolByName(context, 'portal_membership')
            override_setting = self.qup_prefs.object_override
            if override_setting and\
                    mtool.checkPermission(ModifyPortalContent, updated_object):
                can_overwrite = True
            else:
                can_overwrite = False

            if not can_overwrite:
                logger.debug(
                    "The file id for %s already exists, upload rejected"
                    % file_name
                )
                return json.dumps({u'error': u'serverErrorAlreadyExists'})

            overwritten_file = updated_object
        else:
            overwritten_file = None

        content_type = get_content_type(context, file_data, file_name)

        portal_type = getDataFromAllRequests(request, 'typeupload') or ''
        title = getDataFromAllRequests(request, 'title') or ''
        description = getDataFromAllRequests(request, 'description') or ''

        if not portal_type:
            ctr = getToolByName(context, 'content_type_registry')
            portal_type = ctr.findTypeName(
                file_name.lower(), content_type, ''
            ) or 'File'

        if file_data:
            if overwritten_file is not None:
                updater = IQuickUploadFileUpdater(context)
                logger.info(
                    "reuploading %s file with %s: title=%s, description=%s, "
                    "content_type=%s"
                    % (overwritten_file.absolute_url(), upload_with, title,
                       description, content_type))
                try:
                    f = updater(overwritten_file, file_name, title,
                                description, content_type, file_data)
                except Exception, e:
                    logger.error(
                        "Error updating %s file: %s", file_name, str(e)
                    )
                    return json.dumps({u'error': u'serverError'})

            else:
                factory = IQuickUploadFileFactory(context)
                logger.info(
                    "uploading file with %s: filename=%s, title=%s, "
                    "description=%s, content_type=%s, portal_type=%s"
                    % (upload_with, file_name, title,
                       description, content_type, portal_type))
                try:
                    f = factory(file_name, title, description, content_type,
                                file_data, portal_type)
                except Exception, e:
                    logger.error(
                        "Error creating %s file: %s", file_name, str(e)
                    )
                    return json.dumps({u'error': u'serverError'})

            if f['success'] is not None:
                o = f['success']
                logger.info("file url: %s" % o.absolute_url())
                if HAS_UUID:
                    uid = IUUID(o)
                else:
                    uid = o.UID()

                msg = {
                    u'success': True,
                    u'uid': uid,
                    u'name': o.getId(),
                    u'title': o.pretty_title_or_id()
                }
            else:
                msg = {u'error': f['error']}
        else:
            msg = {u'error': u'emptyError'}

        return json.dumps(msg)

    def _check_file_size(self, data):
        max_size = int(self.qup_prefs.size_limit)
        if not max_size:
            return 1

        #file_size = len(data.read()) / 1024
        data.seek(0, os.SEEK_END)
        file_size = data.tell() / 1024
        data.seek(0, os.SEEK_SET)
        max_size = int(self.qup_prefs.size_limit)
        if file_size <= max_size:
            return 1

        return 0


def get_content_type(context, file_data, filename):
    content_type = mimetypes.guess_type(filename)[0]
    # sometimes plone mimetypes registry could be more powerful
    if not content_type:
        mtr = getToolByName(context, 'mimetypes_registry')
        mimetype = mtr.classify(file_data, filename=filename)
        if mimetype is not None:
            return str(mimetype)

    return content_type


class QuickUploadCheckFile(BrowserView):
    """
    check if file exists
    """

    def quick_upload_check_file(self):

        context = aq_inner(self.context)
        request = self.request

        already_exists = {}
        formdict = request.form
        ids = context.objectIds()

        for k, v in formdict.items():
            if k != 'folder':
                if v in ids:
                    already_exists[k] = v

        return str(already_exists)

    def __call__(self):
        """
        """
        return self.quick_upload_check_file()



# This module was directly based on z3c.widget.flashupload.ticket.
# Unfortunately we needed more information, like the username that the ticket
# belongs to so that content can be created with the correct ownership.  And
# the username retrieval is quite Zope2-specific.
from Acquisition import aq_inner
from collective.quickupload import logger
from zope.security.interfaces import Unauthorized

import AccessControl
import random

try:
    from zope.app.cache.ram import RAMCache
except ImportError:
    from zope.ramcache.ram import RAMCache

ticketCache = RAMCache()


def issueTicket(ident):
    """ issues a timelimit ticket
    >>> type(issueTicket(object()))== type('')
    True
    """
    ticket = str(random.random())
    sm = AccessControl.getSecurityManager()
    user = sm.getUser()
    if user is None:
        raise Unauthorized('No currently authenticated user')
    ticketCache.set(user.getId(), ident, key=dict(ticket=ticket))
    return ticket


def validateTicket(ident, ticket):
    """validates a ticket

    >>> validateTicket(object(),'abc')
    False
    >>> obj = object()
    >>> ticket = issueTicket(obj)
    >>> validateTicket(obj,ticket)
    True
    >>> validateTicket(object(),ticket)
    False
    >>> validateTicket(obj,'another')
    False
    """
    ticket = ticketCache.query(ident, dict(ticket=ticket))
    return ticket is not None


def ticketOwner(ident, ticket):
    """Return username of the owner of the ticket.

    >>> ticketOwner(object(), 'abc') is None
    True
    >>> obj = object()
    >>> ticket = issueTicket(obj)
    >>> ticketOwner(obj,ticket)
    'Anonymous User'
    """

    return ticketCache.query(ident, dict(ticket=ticket))


def invalidateTicket(ident, ticket):
    """invalidates a ticket
    >>> ticket = issueTicket(1)
    >>> validateTicket(1,ticket)
    True
    >>> invalidateTicket(1,ticket)
    >>> validateTicket(1,ticket)
    False
    """
    ticketCache.invalidate(ident, dict(ticket=ticket))


class TicketView(object):
    """A view which returns a ticket for its context"""
    
    def __call__(self):
        # IE likes to cache this request, don't let it
        response = self.request.response
        response.setHeader('Cache-Control',
                           'no-cache, no-store, must-revalidate')
        response.setHeader('Pragma', 'no-cache')
        response.setHeader('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')

        logger.debug('Getting ready to issue new ticket')
        context = aq_inner(self.context)
        url = context.absolute_url()
        ticket = issueTicket(url)
        logger.debug('Issued ticket "%s" for url: %s' % (str(ticket), url))

        return ticket



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os


def read(*paths):
    return open(os.path.join(os.path.dirname(__file__), *paths)).read()

version = '1.6.4'

setup(name='collective.quickupload',
      version=version,
      description="Pure javascript files upload tool for Plone, with drag " +
                  "and drop, multi selection, and progress bar.",
      long_description="\n\n".join([
          read("README.rst"),
          read("docs", "HISTORY.rst"),
      ]),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Topic :: Desktop Environment :: File Managers",
          "Programming Language :: Python",
          "Framework :: Plone :: 4.0",
          "Framework :: Plone :: 4.1",
          "Framework :: Plone :: 4.3",
          "Natural Language :: English",
          "Natural Language :: French",
          "Natural Language :: German",
          "Natural Language :: Spanish",
          "Natural Language :: Finnish",
          "Natural Language :: Italian",
          "Natural Language :: Norwegian",
          "Natural Language :: English",
          "Natural Language :: Portuguese (Brazilian)",
          "Natural Language :: Chinese (Simplified)",
          "Natural Language :: Chinese (Traditional)",
          "License :: OSI Approved :: GNU Affero General Public License v3"
      ],
      keywords='Plone Multiple Files Upload',
      author='jean-mat Grimaldi',
      author_email='jeanmat.grimaldi@gmail.com',
      url='http://plone.org/products/collective.quickupload',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'ua_parser',
      ],
      extras_require={
          'test': [
              'plone.app.dexterity',
              'plone.app.testing',
              'mock',
          ],
      },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import unittest2 as unittest
from collective.quickupload.testing import QUICKUPLOAD_INTEGRATION_TESTING
import os.path
from collective.quickupload.browser.quick_upload import get_content_type
from Products.CMFCore.utils import getToolByName

class TestMimetypes(unittest.TestCase):

    layer = QUICKUPLOAD_INTEGRATION_TESTING
    
    def setUp(self):
        self.portal = self.layer['portal']
        file_ = open("%s/testfile_mimetypes.txt" % os.path.split(__file__)[0], 'r')
        self.file_data = file_.read()
        mtr = getToolByName(self.portal, 'mimetypes_registry')
        mtr.manage_addMimeType('Only globs mimetype', ['application/x-only-glob'], [], 'application.png', globs=['*.onlyglob'])
        mtr.manage_addMimeType('Only extension mimetype', ['application/x-only-ext'], ['onlyext'], 'application.png')
    
    def test_only_globs(self):
        content_type = get_content_type(self.portal, self.file_data, 'dummy.onlyglob')
        self.assertEqual('application/x-only-glob', content_type)

    def test_only_ext(self):
        content_type = get_content_type(self.portal, self.file_data, 'dummy.onlyext')
        self.assertEqual('application/x-only-ext', content_type)
    
    def test_recognized_by_python(self):
        content_type = get_content_type(self.portal, self.file_data, 'dummy.txt')
        self.assertEqual('text/plain', content_type)
        